@@ -33,7 +33,10 @@
 #endif
 
 #include "totem-scrsaver.h"
+
+#ifdef WITH_SMCLIENT
 #include "eggsmclient.h"
+#endif
 
 #include "ev-application.h"
 #include "ev-document-factory.h"
@@ -57,6 +60,7 @@ struct _EvApplication {
 	GObject base_instance;
 
 	gchar *dot_dir;
+	gchar *data_dir;
 	gchar *accel_map_file;
 	gchar *toolbars_file;
 	gchar *crashed_file;
@@ -66,9 +70,12 @@ struct _EvApplication {
 
 	TotemScrsaver *scr_saver;
 
+#ifdef WITH_SMCLIENT
 	EggSMClient *smclient;
+#endif
 
-	gchar *last_chooser_uri;
+	gchar *filechooser_open_uri;
+	gchar *filechooser_save_uri;
 
 #ifdef ENABLE_DBUS
 	EvMediaPlayerKeys *keys;
@@ -285,11 +292,14 @@ ev_application_load_session (EvApplication *application)
 	GKeyFile *state_file;
 	gchar   **uri_list;
 
+#ifdef WITH_SMCLIENT
 	if (egg_sm_client_is_resumed (application->smclient)) {
 		state_file = egg_sm_client_get_state_file (application->smclient);
 		if (!state_file)
 			return FALSE;
-	} else if (g_file_test (application->crashed_file, G_FILE_TEST_IS_REGULAR)) {
+	} else
+#endif /* WITH_SMCLIENT */
+        if (g_file_test (application->crashed_file, G_FILE_TEST_IS_REGULAR)) {
 		if (ev_application_run_crash_recovery_dialog (application)) {
 			state_file = g_key_file_new ();
 			g_key_file_load_from_file (state_file,
@@ -323,6 +333,8 @@ ev_application_load_session (EvApplication *application)
 	return TRUE;
 }
 
+#ifdef WITH_SMCLIENT
+
 static void
 smclient_save_state_cb (EggSMClient   *client,
 			GKeyFile      *state_file,
@@ -344,12 +356,15 @@ smclient_quit_cb (EggSMClient   *client,
 	ev_application_shutdown (application);
 }
 
+#endif /* WITH_SMCLIENT */
+
 static void
 ev_application_init_session (EvApplication *application)
 {
 	application->crashed_file = g_build_filename (application->dot_dir,
 						      "evince-crashed", NULL);
 
+#ifdef WITH_SMCLIENT
 	application->smclient = egg_sm_client_get ();
 	g_signal_connect (application->smclient, "save_state",
 			  G_CALLBACK (smclient_save_state_cb),
@@ -357,6 +372,7 @@ ev_application_init_session (EvApplication *application)
 	g_signal_connect (application->smclient, "quit",
 			  G_CALLBACK (smclient_quit_cb),
 			  application);
+#endif
 }
 
 /**
@@ -786,8 +802,12 @@ ev_application_shutdown (EvApplication *application)
 
         g_free (application->dot_dir);
         application->dot_dir = NULL;
-	g_free (application->last_chooser_uri);
-        application->last_chooser_uri = NULL;
+        g_free (application->data_dir);
+        application->data_dir = NULL;
+	g_free (application->filechooser_open_uri);
+        application->filechooser_open_uri = NULL;
+	g_free (application->filechooser_save_uri);
+	application->filechooser_save_uri = NULL;
 
 	g_object_unref (application);
         instance = NULL;
@@ -805,6 +825,7 @@ ev_application_init (EvApplication *ev_application)
 {
 	gint i;
 	const gchar *home_dir;
+	gchar *toolbar_path;
 
         ev_application->dot_dir = g_build_filename (g_get_home_dir (),
                                                     ".gnome2",
@@ -815,6 +836,18 @@ ev_application_init (EvApplication *ev_application)
         if (!ev_dir_ensure_exists (ev_application->dot_dir, 0700))
                 exit (1);
 
+#ifdef G_OS_WIN32
+{
+	gchar *dir;
+
+	dir = g_win32_get_package_installation_directory_of_module (NULL);
+	ev_application->data_dir = g_build_filename (dir, "share", "evince", NULL);
+	g_free (dir);
+}
+#else
+	ev_application->data_dir = g_strdup (DATADIR);
+#endif
+
 	ev_application_init_session (ev_application);
 
 	home_dir = g_get_home_dir ();
@@ -832,14 +865,17 @@ ev_application_init (EvApplication *ev_application)
 	ev_application->toolbars_file = g_build_filename
 			(ev_application->dot_dir, "evince_toolbar.xml", NULL);
 
+	toolbar_path = g_build_filename (ev_application->data_dir,
+					 "evince-toolbar.xml", NULL);
 	egg_toolbars_model_load_names (ev_application->toolbars_model,
-				       DATADIR "/evince-toolbar.xml");
+				       toolbar_path);
 
 	if (!egg_toolbars_model_load_toolbars (ev_application->toolbars_model,
 					       ev_application->toolbars_file)) {
 		egg_toolbars_model_load_toolbars (ev_application->toolbars_model,
-						  DATADIR"/evince-toolbar.xml");
+						  toolbar_path);
 	}
+	g_free (toolbar_path);
 
 	/* Open item doesn't exist anymore,
 	 * convert it to OpenRecent for compatibility
@@ -924,16 +960,32 @@ ev_application_save_toolbars_model (EvApplication *application)
 }
 
 void
-ev_application_set_chooser_uri (EvApplication *application, const gchar *uri)
+ev_application_set_filechooser_uri (EvApplication       *application,
+				    GtkFileChooserAction action,
+				    const gchar         *uri)
 {
-	g_free (application->last_chooser_uri);
-	application->last_chooser_uri = g_strdup (uri);
+	if (action == GTK_FILE_CHOOSER_ACTION_OPEN) {
+		g_free (application->filechooser_open_uri);
+		application->filechooser_open_uri = g_strdup (uri);
+	} else if (action == GTK_FILE_CHOOSER_ACTION_SAVE) {
+		g_free (application->filechooser_save_uri);
+		application->filechooser_save_uri = g_strdup (uri);
+	}
 }
 
 const gchar *
-ev_application_get_chooser_uri (EvApplication *application)
+ev_application_get_filechooser_uri (EvApplication       *application,
+				    GtkFileChooserAction action)
 {
-	return application->last_chooser_uri;
+	if (action == GTK_FILE_CHOOSER_ACTION_OPEN) {
+		if (application->filechooser_open_uri)
+			return application->filechooser_open_uri;
+	} else if (action == GTK_FILE_CHOOSER_ACTION_SAVE) {
+		if (application->filechooser_save_uri)
+			return application->filechooser_save_uri;
+	}
+
+	return NULL;
 }
 
 void
@@ -1095,3 +1147,9 @@ ev_application_get_dot_dir (EvApplication   *application)
 {
 	return application->dot_dir;
 }
+
+const gchar *
+ev_application_get_data_dir (EvApplication   *application)
+{
+	return application->data_dir;
+}
